@@ -22,7 +22,7 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 {
 	QHBoxLayout *hlayout;
 	signal_handler_t *handler = obs_source_get_signal_handler(source);
-	const char *sourceName = obs_source_get_name(source);
+	QString sourceName = QT_UTF8(obs_source_get_name(source));
 	float vol = obs_source_get_volume(source);
 	uint32_t flags = obs_source_get_flags(source);
 	uint32_t mixers = obs_source_get_audio_mixers(source);
@@ -90,7 +90,7 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 	iconLabel->setFixedSize(16, 16);
 	iconLabel->setStyleSheet("background: none");
 
-	nameLabel->setText(QT_UTF8(sourceName));
+	nameLabel->setText(sourceName);
 	nameLabel->setAlignment(Qt::AlignVCenter);
 
 	bool isActive = obs_source_active(source);
@@ -100,7 +100,7 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 		setThemeID(active, "error");
 	activeContainer->layout()->addWidget(active);
 	activeContainer->layout()->setAlignment(active, Qt::AlignVCenter);
-	activeContainer->setFixedWidth(50);
+	activeContainer->setFixedWidth(120);
 
 	volume->setMinimum(MIN_DB - 0.1);
 	volume->setMaximum(MAX_DB);
@@ -109,15 +109,21 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 	volume->setSuffix(" dB");
 	volume->setValue(obs_mul_to_db(vol));
 	volume->setFixedWidth(100);
+	volume->setAccessibleName(
+		QTStr("Basic.AdvAudio.VolumeSource").arg(sourceName));
 
-	if (volume->value() < MIN_DB)
+	if (volume->value() < MIN_DB) {
 		volume->setSpecialValueText("-inf dB");
+		volume->setAccessibleDescription("-inf dB");
+	}
 
 	percent->setMinimum(0);
 	percent->setMaximum(2000);
 	percent->setSuffix("%");
 	percent->setValue((int)(obs_source_get_volume(source) * 100.0f));
 	percent->setFixedWidth(100);
+	percent->setAccessibleName(
+		QTStr("Basic.AdvAudio.VolumeSource").arg(sourceName));
 
 	stackedWidget->addWidget(volume);
 	stackedWidget->addWidget(percent);
@@ -128,6 +134,8 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 	SetVolumeWidget(volType);
 
 	forceMono->setChecked((flags & OBS_SOURCE_FLAG_FORCE_MONO) != 0);
+	forceMono->setAccessibleName(
+		QTStr("Basic.AdvAudio.MonoSource").arg(sourceName));
 
 	forceMonoContainer->layout()->addWidget(forceMono);
 	forceMonoContainer->layout()->setAlignment(forceMono, Qt::AlignVCenter);
@@ -138,6 +146,8 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 	balance->setMaximum(100);
 	balance->setTickPosition(QSlider::TicksAbove);
 	balance->setTickInterval(50);
+	balance->setAccessibleName(
+		QTStr("Basic.AdvAudio.BalanceSource").arg(sourceName));
 
 	const char *speakers =
 		config_get_string(main->Config(), "Audio", "ChannelSetup");
@@ -156,6 +166,8 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 	syncOffset->setSuffix(" ms");
 	syncOffset->setValue(int(cur_sync / NSEC_PER_MSEC));
 	syncOffset->setFixedWidth(100);
+	syncOffset->setAccessibleName(
+		QTStr("Basic.AdvAudio.SyncOffsetSource").arg(sourceName));
 
 	int idx;
 #if defined(_WIN32) || defined(__APPLE__) || HAVE_PULSEAUDIO
@@ -168,20 +180,34 @@ OBSAdvAudioCtrl::OBSAdvAudioCtrl(QGridLayout *, obs_source_t *source_)
 	int mt = (int)obs_source_get_monitoring_type(source);
 	idx = monitoringType->findData(mt);
 	monitoringType->setCurrentIndex(idx);
+	monitoringType->setAccessibleName(
+		QTStr("Basic.AdvAudio.MonitoringSource").arg(sourceName));
 #endif
 
 	mixer1->setText("1");
 	mixer1->setChecked(mixers & (1 << 0));
+	mixer1->setAccessibleName(
+		QTStr("Basic.Settings.Output.Adv.Audio.Track1"));
 	mixer2->setText("2");
 	mixer2->setChecked(mixers & (1 << 1));
+	mixer2->setAccessibleName(
+		QTStr("Basic.Settings.Output.Adv.Audio.Track2"));
 	mixer3->setText("3");
 	mixer3->setChecked(mixers & (1 << 2));
+	mixer3->setAccessibleName(
+		QTStr("Basic.Settings.Output.Adv.Audio.Track3"));
 	mixer4->setText("4");
 	mixer4->setChecked(mixers & (1 << 3));
+	mixer4->setAccessibleName(
+		QTStr("Basic.Settings.Output.Adv.Audio.Track4"));
 	mixer5->setText("5");
 	mixer5->setChecked(mixers & (1 << 4));
+	mixer5->setAccessibleName(
+		QTStr("Basic.Settings.Output.Adv.Audio.Track5"));
 	mixer6->setText("6");
 	mixer6->setChecked(mixers & (1 << 5));
+	mixer6->setAccessibleName(
+		QTStr("Basic.Settings.Output.Adv.Audio.Track6"));
 
 	speaker_layout sl = obs_source_get_speaker_layout(source);
 
@@ -351,7 +377,9 @@ void OBSAdvAudioCtrl::SourceVolumeChanged(float value)
 
 void OBSAdvAudioCtrl::SourceSyncChanged(int64_t offset)
 {
+	syncOffset->blockSignals(true);
 	syncOffset->setValue(offset / NSEC_PER_MSEC);
+	syncOffset->blockSignals(false);
 }
 
 void OBSAdvAudioCtrl::SourceMixersChanged(uint32_t mixers)
@@ -369,6 +397,8 @@ void OBSAdvAudioCtrl::SourceMixersChanged(uint32_t mixers)
 
 void OBSAdvAudioCtrl::volumeChanged(double db)
 {
+	float prev = obs_source_get_volume(source);
+
 	if (db < MIN_DB) {
 		volume->setSpecialValueText("-inf dB");
 		db = -INFINITY;
@@ -376,30 +406,87 @@ void OBSAdvAudioCtrl::volumeChanged(double db)
 
 	float val = obs_db_to_mul(db);
 	obs_source_set_volume(source, val);
+
+	auto undo_redo = [](const std::string &name, float val) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		obs_source_set_volume(source, val);
+		obs_source_release(source);
+	};
+
+	const char *name = obs_source_get_name(source);
+
+	OBSBasic *main = OBSBasic::Get();
+	main->undo_s.add_action(
+		QTStr("Undo.Volume.Change").arg(name),
+		std::bind(undo_redo, std::placeholders::_1, prev),
+		std::bind(undo_redo, std::placeholders::_1, val), name, name,
+		true);
 }
 
 void OBSAdvAudioCtrl::percentChanged(int percent)
 {
-	obs_source_set_volume(source, (float)percent / 100.0f);
+	float prev = obs_source_get_volume(source);
+	float val = (float)percent / 100.0f;
+
+	obs_source_set_volume(source, val);
+
+	auto undo_redo = [](const std::string &name, float val) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		obs_source_set_volume(source, val);
+		obs_source_release(source);
+	};
+
+	const char *name = obs_source_get_name(source);
+	OBSBasic::Get()->undo_s.add_action(
+		QTStr("Undo.Volume.Change").arg(name),
+		std::bind(undo_redo, std::placeholders::_1, prev),
+		std::bind(undo_redo, std::placeholders::_1, val), name, name,
+		true);
 }
 
-void OBSAdvAudioCtrl::downmixMonoChanged(bool checked)
+static inline void set_mono(obs_source_t *source, bool mono)
+{
+	uint32_t flags = obs_source_get_flags(source);
+	if (mono)
+		flags |= OBS_SOURCE_FLAG_FORCE_MONO;
+	else
+		flags &= ~OBS_SOURCE_FLAG_FORCE_MONO;
+	obs_source_set_flags(source, flags);
+}
+
+void OBSAdvAudioCtrl::downmixMonoChanged(bool val)
 {
 	uint32_t flags = obs_source_get_flags(source);
 	bool forceMonoActive = (flags & OBS_SOURCE_FLAG_FORCE_MONO) != 0;
 
-	if (forceMonoActive != checked) {
-		if (checked)
-			flags |= OBS_SOURCE_FLAG_FORCE_MONO;
-		else
-			flags &= ~OBS_SOURCE_FLAG_FORCE_MONO;
+	if (forceMonoActive == val)
+		return;
 
-		obs_source_set_flags(source, flags);
-	}
+	if (val)
+		flags |= OBS_SOURCE_FLAG_FORCE_MONO;
+	else
+		flags &= ~OBS_SOURCE_FLAG_FORCE_MONO;
+
+	obs_source_set_flags(source, flags);
+
+	auto undo_redo = [](const std::string &name, bool val) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		set_mono(source, val);
+		obs_source_release(source);
+	};
+
+	QString text = QTStr(val ? "Undo.ForceMono.On" : "Undo.ForceMono.Off");
+
+	const char *name = obs_source_get_name(source);
+	OBSBasic::Get()->undo_s.add_action(
+		text.arg(name),
+		std::bind(undo_redo, std::placeholders::_1, !val),
+		std::bind(undo_redo, std::placeholders::_1, val), name, name);
 }
 
 void OBSAdvAudioCtrl::balanceChanged(int val)
 {
+	float prev = obs_source_get_balance_value(source);
 	float bal = (float)val / 100.0f;
 
 	if (abs(50 - val) < 10) {
@@ -410,6 +497,19 @@ void OBSAdvAudioCtrl::balanceChanged(int val)
 	}
 
 	obs_source_set_balance_value(source, bal);
+
+	auto undo_redo = [](const std::string &name, float val) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		obs_source_set_balance_value(source, val);
+		obs_source_release(source);
+	};
+
+	const char *name = obs_source_get_name(source);
+	OBSBasic::Get()->undo_s.add_action(
+		QTStr("Undo.Balance.Change").arg(name),
+		std::bind(undo_redo, std::placeholders::_1, prev),
+		std::bind(undo_redo, std::placeholders::_1, bal), name, name,
+		true);
 }
 
 void OBSAdvAudioCtrl::ResetBalance()
@@ -419,17 +519,35 @@ void OBSAdvAudioCtrl::ResetBalance()
 
 void OBSAdvAudioCtrl::syncOffsetChanged(int milliseconds)
 {
-	int64_t cur_val = obs_source_get_sync_offset(source);
+	int64_t prev = obs_source_get_sync_offset(source);
+	int64_t val = int64_t(milliseconds) * NSEC_PER_MSEC;
 
-	if (cur_val / NSEC_PER_MSEC != milliseconds)
-		obs_source_set_sync_offset(source, int64_t(milliseconds) *
-							   NSEC_PER_MSEC);
+	if (prev / NSEC_PER_MSEC == milliseconds)
+		return;
+
+	obs_source_set_sync_offset(source, val);
+
+	auto undo_redo = [](const std::string &name, int64_t val) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		obs_source_set_sync_offset(source, val);
+		obs_source_release(source);
+	};
+
+	const char *name = obs_source_get_name(source);
+	OBSBasic::Get()->undo_s.add_action(
+		QTStr("Undo.SyncOffset.Change").arg(name),
+		std::bind(undo_redo, std::placeholders::_1, prev),
+		std::bind(undo_redo, std::placeholders::_1, val), name, name,
+		true);
 }
 
 void OBSAdvAudioCtrl::monitoringTypeChanged(int index)
 {
-	int mt = monitoringType->itemData(index).toInt();
-	obs_source_set_monitoring_type(source, (obs_monitoring_type)mt);
+	obs_monitoring_type prev = obs_source_get_monitoring_type(source);
+
+	obs_monitoring_type mt =
+		(obs_monitoring_type)monitoringType->itemData(index).toInt();
+	obs_source_set_monitoring_type(source, mt);
 
 	const char *type = nullptr;
 
@@ -445,8 +563,20 @@ void OBSAdvAudioCtrl::monitoringTypeChanged(int index)
 		break;
 	}
 
+	const char *name = obs_source_get_name(source);
 	blog(LOG_INFO, "User changed audio monitoring for source '%s' to: %s",
-	     obs_source_get_name(source), type);
+	     name ? name : "(null)", type);
+
+	auto undo_redo = [](const std::string &name, obs_monitoring_type val) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		obs_source_set_monitoring_type(source, val);
+		obs_source_release(source);
+	};
+
+	OBSBasic::Get()->undo_s.add_action(
+		QTStr("Undo.MonitoringType.Change").arg(name),
+		std::bind(undo_redo, std::placeholders::_1, prev),
+		std::bind(undo_redo, std::placeholders::_1, mt), name, name);
 }
 
 static inline void setMixer(obs_source_t *source, const int mixerIdx,
@@ -461,6 +591,19 @@ static inline void setMixer(obs_source_t *source, const int mixerIdx,
 		new_mixers &= ~(1 << mixerIdx);
 
 	obs_source_set_audio_mixers(source, new_mixers);
+
+	auto undo_redo = [](const std::string &name, uint32_t mixers) {
+		obs_source_t *source = obs_get_source_by_name(name.c_str());
+		obs_source_set_audio_mixers(source, mixers);
+		obs_source_release(source);
+	};
+
+	const char *name = obs_source_get_name(source);
+	OBSBasic::Get()->undo_s.add_action(
+		QTStr("Undo.Mixers.Change").arg(name),
+		std::bind(undo_redo, std::placeholders::_1, mixers),
+		std::bind(undo_redo, std::placeholders::_1, new_mixers), name,
+		name);
 }
 
 void OBSAdvAudioCtrl::mixer1Changed(bool checked)
