@@ -110,6 +110,7 @@ static bool get_client_box(HWND window, uint32_t width, uint32_t height,
 struct winrt_capture {
 	HWND window;
 	bool client_area;
+	HMONITOR monitor;
 
 	bool capture_cursor;
 	BOOL cursor_visible;
@@ -186,8 +187,10 @@ struct winrt_capture {
 			}
 
 			if (!texture) {
-				texture = gs_texture_create_gdi(texture_width,
-								texture_height);
+				texture = gs_texture_create(texture_width,
+							    texture_height,
+							    GS_BGRA, 1, NULL,
+							    0);
 			}
 
 			if (client_area) {
@@ -230,8 +233,27 @@ static void winrt_capture_device_loss_release(void *data)
 	capture->active = FALSE;
 
 	capture->frame_arrived.revoke();
-	capture->frame_pool.Close();
-	capture->session.Close();
+
+	try {
+		capture->frame_pool.Close();
+	} catch (winrt::hresult_error &err) {
+		blog(LOG_ERROR,
+		     "Direct3D11CaptureFramePool::Close (0x%08X): %ls",
+		     err.to_abi(), err.message().c_str());
+	} catch (...) {
+		blog(LOG_ERROR, "Direct3D11CaptureFramePool::Close (0x%08X)",
+		     winrt::to_hresult());
+	}
+
+	try {
+		capture->session.Close();
+	} catch (winrt::hresult_error &err) {
+		blog(LOG_ERROR, "GraphicsCaptureSession::Close (0x%08X): %ls",
+		     err.to_abi(), err.message().c_str());
+	} catch (...) {
+		blog(LOG_ERROR, "GraphicsCaptureSession::Close (0x%08X)",
+		     winrt::to_hresult());
+	}
 
 	capture->session = nullptr;
 	capture->frame_pool = nullptr;
@@ -240,6 +262,71 @@ static void winrt_capture_device_loss_release(void *data)
 	capture->item = nullptr;
 }
 
+#ifdef NTDDI_WIN10_FE
+static bool winrt_capture_border_toggle_supported()
+try {
+	return winrt::Windows::Foundation::Metadata::ApiInformation::
+		IsPropertyPresent(
+			L"Windows.Graphics.Capture.GraphicsCaptureSession",
+			L"IsBorderRequired");
+} catch (const winrt::hresult_error &err) {
+	blog(LOG_ERROR, "winrt_capture_border_toggle_supported (0x%08X): %ls",
+	     err.to_abi(), err.message().c_str());
+	return false;
+} catch (...) {
+	blog(LOG_ERROR, "winrt_capture_border_toggle_supported (0x%08X)",
+	     winrt::to_hresult());
+	return false;
+}
+#endif
+
+static winrt::Windows::Graphics::Capture::GraphicsCaptureItem
+winrt_capture_create_item(IGraphicsCaptureItemInterop *const interop_factory,
+			  HWND window, HMONITOR monitor)
+{
+	winrt::Windows::Graphics::Capture::GraphicsCaptureItem item = {nullptr};
+	if (window) {
+		try {
+			const HRESULT hr = interop_factory->CreateForWindow(
+				window,
+				winrt::guid_of<ABI::Windows::Graphics::Capture::
+						       IGraphicsCaptureItem>(),
+				reinterpret_cast<void **>(
+					winrt::put_abi(item)));
+			if (FAILED(hr))
+				blog(LOG_ERROR, "CreateForWindow (0x%08X)", hr);
+		} catch (winrt::hresult_error &err) {
+			blog(LOG_ERROR, "CreateForWindow (0x%08X): %ls",
+			     err.to_abi(), err.message().c_str());
+		} catch (...) {
+			blog(LOG_ERROR, "CreateForWindow (0x%08X)",
+			     winrt::to_hresult());
+		}
+	} else {
+		assert(monitor);
+
+		try {
+			const HRESULT hr = interop_factory->CreateForMonitor(
+				monitor,
+				winrt::guid_of<ABI::Windows::Graphics::Capture::
+						       IGraphicsCaptureItem>(),
+				reinterpret_cast<void **>(
+					winrt::put_abi(item)));
+			if (FAILED(hr))
+				blog(LOG_ERROR, "CreateForMonitor (0x%08X)",
+				     hr);
+		} catch (winrt::hresult_error &err) {
+			blog(LOG_ERROR, "CreateForMonitor (0x%08X): %ls",
+			     err.to_abi(), err.message().c_str());
+		} catch (...) {
+			blog(LOG_ERROR, "CreateForMonitor (0x%08X)",
+			     winrt::to_hresult());
+		}
+	}
+
+	return item;
+}
+
 static void winrt_capture_device_loss_rebuild(void *device_void, void *data)
 {
 	winrt_capture *capture = static_cast<winrt_capture *>(data);
@@ -248,20 +335,9 @@ static void winrt_capture_device_loss_rebuild(void *device_void, void *data)
 		winrt::Windows::Graphics::Capture::GraphicsCaptureItem>();
 	auto interop_factory =
 		activation_factory.as<IGraphicsCaptureItemInterop>();
-	winrt::Windows::Graphics::Capture::GraphicsCaptureItem item = {nullptr};
-	try {
-		interop_factory->CreateForWindow(
-			capture->window,
-			winrt::guid_of<ABI::Windows::Graphics::Capture::
-					       IGraphicsCaptureItem>(),
-			reinterpret_cast<void **>(winrt::put_abi(item)));
-	} catch (winrt::hresult_error &err) {
-		blog(LOG_ERROR, "CreateForWindow (0x%08X): %ls", err.to_abi(),
-		     err.message().c_str());
-	} catch (...) {
-		blog(LOG_ERROR, "CreateForWindow (0x%08X)",
-		     winrt::to_hresult());
-	}
+	winrt::Windows::Graphics::Capture::GraphicsCaptureItem item =
+		winrt_capture_create_item(interop_factory.get(),
+					  capture->window, capture->monitor);
 
 	ID3D11Device *const d3d_device = (ID3D11Device *)device_void;
 	ComPtr<IDXGIDevice> dxgi_device;
@@ -286,6 +362,17 @@ static void winrt_capture_device_loss_rebuild(void *device_void, void *data)
 	const winrt::Windows::Graphics::Capture::GraphicsCaptureSession session =
 		frame_pool.CreateCaptureSession(item);
 
+#ifdef NTDDI_WIN10_FE
+	if (winrt_capture_border_toggle_supported()) {
+		winrt::Windows::Graphics::Capture::GraphicsCaptureAccess::
+			RequestAccessAsync(
+				winrt::Windows::Graphics::Capture::
+					GraphicsCaptureAccessKind::Borderless)
+				.get();
+		session.IsBorderRequired(false);
+	}
+#endif
+
 	if (winrt_capture_cursor_toggle_supported())
 		session.IsCursorCaptureEnabled(capture->capture_cursor &&
 					       capture->cursor_visible);
@@ -310,8 +397,10 @@ static void winrt_capture_device_loss_rebuild(void *device_void, void *data)
 	}
 }
 
-extern "C" EXPORT struct winrt_capture *
-winrt_capture_init(BOOL cursor, HWND window, BOOL client_area)
+static struct winrt_capture *winrt_capture_init_internal(BOOL cursor,
+							 HWND window,
+							 BOOL client_area,
+							 HMONITOR monitor)
 try {
 	ID3D11Device *const d3d_device = (ID3D11Device *)gs_get_device_obj();
 	ComPtr<IDXGIDevice> dxgi_device;
@@ -334,22 +423,11 @@ try {
 		winrt::Windows::Graphics::Capture::GraphicsCaptureItem>();
 	auto interop_factory =
 		activation_factory.as<IGraphicsCaptureItemInterop>();
-	winrt::Windows::Graphics::Capture::GraphicsCaptureItem item = {nullptr};
-	try {
-		interop_factory->CreateForWindow(
-			window,
-			winrt::guid_of<ABI::Windows::Graphics::Capture::
-					       IGraphicsCaptureItem>(),
-			reinterpret_cast<void **>(winrt::put_abi(item)));
-	} catch (winrt::hresult_error &err) {
-		blog(LOG_ERROR, "CreateForWindow (0x%08X): %ls", err.to_abi(),
-		     err.message().c_str());
+	winrt::Windows::Graphics::Capture::GraphicsCaptureItem item =
+		winrt_capture_create_item(interop_factory.get(), window,
+					  monitor);
+	if (!item)
 		return nullptr;
-	} catch (...) {
-		blog(LOG_ERROR, "CreateForWindow (0x%08X)",
-		     winrt::to_hresult());
-		return nullptr;
-	}
 
 	const winrt::Windows::Graphics::DirectX::Direct3D11::IDirect3DDevice
 		device = inspectable.as<winrt::Windows::Graphics::DirectX::
@@ -365,6 +443,17 @@ try {
 	const winrt::Windows::Graphics::Capture::GraphicsCaptureSession session =
 		frame_pool.CreateCaptureSession(item);
 
+#ifdef NTDDI_WIN10_FE
+	if (winrt_capture_border_toggle_supported()) {
+		winrt::Windows::Graphics::Capture::GraphicsCaptureAccess::
+			RequestAccessAsync(
+				winrt::Windows::Graphics::Capture::
+					GraphicsCaptureAccessKind::Borderless)
+				.get();
+		session.IsBorderRequired(false);
+	}
+#endif
+
 	/* disable cursor capture if possible since ours performs better */
 	const BOOL cursor_toggle_supported =
 		winrt_capture_cursor_toggle_supported();
@@ -374,6 +463,7 @@ try {
 	struct winrt_capture *capture = new winrt_capture{};
 	capture->window = window;
 	capture->client_area = client_area;
+	capture->monitor = monitor;
 	capture->capture_cursor = cursor && cursor_toggle_supported;
 	capture->cursor_visible = cursor;
 	capture->item = item;
@@ -410,6 +500,18 @@ try {
 	return nullptr;
 }
 
+extern "C" EXPORT struct winrt_capture *
+winrt_capture_init_window(BOOL cursor, HWND window, BOOL client_area)
+{
+	return winrt_capture_init_internal(cursor, window, client_area, NULL);
+}
+
+extern "C" EXPORT struct winrt_capture *
+winrt_capture_init_monitor(BOOL cursor, HMONITOR monitor)
+{
+	return winrt_capture_init_internal(cursor, NULL, false, monitor);
+}
+
 extern "C" EXPORT void winrt_capture_free(struct winrt_capture *capture)
 {
 	if (capture) {
@@ -433,8 +535,30 @@ extern "C" EXPORT void winrt_capture_free(struct winrt_capture *capture)
 
 		capture->frame_arrived.revoke();
 		capture->closed.revoke();
-		capture->frame_pool.Close();
-		capture->session.Close();
+
+		try {
+			capture->frame_pool.Close();
+		} catch (winrt::hresult_error &err) {
+			blog(LOG_ERROR,
+			     "Direct3D11CaptureFramePool::Close (0x%08X): %ls",
+			     err.to_abi(), err.message().c_str());
+		} catch (...) {
+			blog(LOG_ERROR,
+			     "Direct3D11CaptureFramePool::Close (0x%08X)",
+			     winrt::to_hresult());
+		}
+
+		try {
+			capture->session.Close();
+		} catch (winrt::hresult_error &err) {
+			blog(LOG_ERROR,
+			     "GraphicsCaptureSession::Close (0x%08X): %ls",
+			     err.to_abi(), err.message().c_str());
+		} catch (...) {
+			blog(LOG_ERROR,
+			     "GraphicsCaptureSession::Close (0x%08X)",
+			     winrt::to_hresult());
+		}
 
 		delete capture;
 	}
@@ -447,7 +571,15 @@ static void draw_texture(struct winrt_capture *capture, gs_effect_t *effect)
 	gs_eparam_t *image = gs_effect_get_param_by_name(effect, "image");
 	size_t passes;
 
-	gs_effect_set_texture(image, texture);
+	const bool linear_srgb = gs_get_linear_srgb();
+
+	const bool previous = gs_framebuffer_srgb_enabled();
+	gs_enable_framebuffer_srgb(linear_srgb);
+
+	if (linear_srgb)
+		gs_effect_set_texture_srgb(image, texture);
+	else
+		gs_effect_set_texture(image, texture);
 
 	passes = gs_technique_begin(tech);
 	for (size_t i = 0; i < passes; i++) {
@@ -458,6 +590,8 @@ static void draw_texture(struct winrt_capture *capture, gs_effect_t *effect)
 		}
 	}
 	gs_technique_end(tech);
+
+	gs_enable_framebuffer_srgb(previous);
 }
 
 extern "C" EXPORT BOOL winrt_capture_active(const struct winrt_capture *capture)
@@ -465,15 +599,32 @@ extern "C" EXPORT BOOL winrt_capture_active(const struct winrt_capture *capture)
 	return capture->active;
 }
 
-extern "C" EXPORT void winrt_capture_show_cursor(struct winrt_capture *capture,
+extern "C" EXPORT BOOL winrt_capture_show_cursor(struct winrt_capture *capture,
 						 BOOL visible)
 {
-	if (capture->capture_cursor) {
-		if (capture->cursor_visible != visible) {
-			capture->session.IsCursorCaptureEnabled(visible);
-			capture->cursor_visible = visible;
+	BOOL success = FALSE;
+
+	try {
+		if (capture->capture_cursor) {
+			if (capture->cursor_visible != visible) {
+				capture->session.IsCursorCaptureEnabled(
+					visible);
+				capture->cursor_visible = visible;
+			}
 		}
+
+		success = TRUE;
+	} catch (winrt::hresult_error &err) {
+		blog(LOG_ERROR,
+		     "GraphicsCaptureSession::IsCursorCaptureEnabled (0x%08X): %ls",
+		     err.to_abi(), err.message().c_str());
+	} catch (...) {
+		blog(LOG_ERROR,
+		     "GraphicsCaptureSession::IsCursorCaptureEnabled (0x%08X)",
+		     winrt::to_hresult());
 	}
+
+	return success;
 }
 
 extern "C" EXPORT void winrt_capture_render(struct winrt_capture *capture,
